<?php

function entity_reports_admin_overview() {
  $ret = [];
  $ret['title'] = ['#markup' => '<h1>Entity reports overview</h1>'];
  $ret['description'] = ['#markup' => '<p>Hover entity name to see its description. Click on bundles to see their structure</p>'];

  $entities = entity_get_info();
  ksort($entities);
  $rows = [];
  $base_url_bundle = 'admin/reports/entity';
  foreach ($entities as $machine_name => $entity) {
    $row = [];
    if (!empty($entity['description'])) {
      $row[] = sprintf('<div title="%s">%s</div>', $entity['description'], $entity['label']);
    }
    else {
      $row[] = $entity['label'];
    }
    $row[] = $machine_name;
    $row[] = !empty($entity['fieldable']) ? 'Yes' : '';
    $bundles = [];
    foreach ($entity['bundles'] as $bkey => $bundle) {
      $bundles[] = l($bundle['label'], $base_url_bundle . '/' . $machine_name . '/' . $bkey);
    }
    $row[] = implode(', ', $bundles);
    $rows[] = $row;
  }
  $ret['overview'] = [
    '#theme' => 'table',
    '#header' => [
      'Entity name',
      'Machine name',
      'Fieldable',
      'Bundles',
    ],
    '#rows' => $rows,
  ];
  return $ret;
}

function entity_reports_entity_bundle_overview($entity_name, $bundle_name) {
  $ret = [];
  $entity = entity_get_info($entity_name);
  $bundle = $entity['bundles'][$bundle_name];
  $ret['title'] = ['#markup' => '<h1>' . $entity['label'] . ' &raquo; ' . $bundle['label'] . '</h1>'];
  $api = new EntityReport($entity_name, $bundle_name);

  $rows = [];

  $rows[] = [
    'Name',
    $bundle['label'],
  ];
  $rows[] = [
    t('Machine name'),
    $bundle_name,
  ];

  if ($entity_name == 'node') {
    $node_type = node_type_load($bundle_name);
    $rows[] = [
      t('Description'),
      $node_type->description
    ];
  }

  $count = $api->getTotalCount();
  $rows[] = ['Total entities', sprintf('%d %s', $count, $api->getPluralLabel())];
  try {
    $rows[] = [
      'Published entities',
      sprintf('%d %s', $count, $api->getPluralLabel())
    ];
  }
  catch (Exception $e) {
  }

  $translation = $api->isTranslatableLabel();
  $rows[] = [
    'Translatable',
    $translation,
  ];

  if (!empty($entity['fieldable'])) {
    $rows[] = [
      'View field structure',
      l('view', $bundle['admin']['real path'] . '/fields'),
    ];
  }

  $ret['overview'] = [
    '#theme' => 'table',
    '#header' => [
      'Key',
      'Value',
    ],
    '#rows' => $rows,
  ];

  $field_types = field_info_field_types();

  if (!empty($entity['fieldable'])) {
    $rows = [];
    $fields = $api->getFieldInformation();
    uasort($fields, function ($a, $b) {
      return strcmp($a['label'], $b['label']);
    });
    foreach ($fields as $name => $instance) {
      $info = field_info_field($name);
      $row = [];
      $row[] = $instance['label'];
      $row[] = $name;
      $row[] = $api->getFieldTypeLabel($name);
      $row[] = $instance['description'];
      $row[] = $info['cardinality'];
      $row[] = empty($instance['required']) ? 'No' : 'Yes';
      $row[] = empty($info['translatable']) ? 'No' : 'Yes';
      $row[] = $api->getFieldValueExample($name, FALSE);
      $row[] = $api->getFieldValueExample($name, TRUE);
      $rows[] = $row;
    }
    $ret['fields'] = [
      '#prefix' => '<h2>' . t('Field information') . '</h2>',
      '#theme' => 'table',
      '#header' => [
        'Field name',
        'Machine name',
        'Type',
        'Description',
        'Cardinality',
        'Required',
        'Translatable',
        'Original language example',
        'Translation example',
      ],
      '#rows' => $rows,
    ];
  }

  return $ret;
}